local frame = CreateFrame('Frame', 'KeepSakeFrame')
frame:Hide()

-- pull the frame in from the edges of the screen a little
local framePaddingTop = 8
local framePaddingSides = 10
frame:SetPoint('TOPLEFT', UIParent, 'TOPLEFT', framePaddingSides, -framePaddingTop)
frame:SetPoint('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', -framePaddingSides, framePaddingTop)

function createFrameText(inherits)
    local text = frame:CreateFontString(nil, nil, inherits)
    text:SetTextColor(1, 0.8, 0, 1)
    text:SetText('')
    return text
end

-- for the various refresh functions, return true if there was something useful
-- to display. this is used by the main loop in order to make all the text stack
-- snugly without gaps. for example, not everyone is in a guild. so, we don't
-- want a gap between your name and level.

function refreshName(text)
    text:SetText(UnitName('player') .. ' of ' .. GetRealmName())
    return true
end

function refreshGuild(text)
    local currentGuildName, currentRankName = GetGuildInfo('player')
    if currentGuildName ~= nil then
        text:SetText(currentRankName .. ' of <' .. currentGuildName .. '>')
        return true
    end
    return false
end

function refreshLevel(text)
    local currentPercentXP = floor(UnitXP('player') / UnitXPMax('player') * 100)
    text:SetText('Level: ' .. UnitLevel('player') .. (currentPercentXP > 0 and ' XP: ' .. currentPercentXP .. '%' or ''))
    return true
end

function refreshZone(text)
    local currentZone = GetRealZoneText()
    local _, _, currentInstanceDifficulty = GetInstanceInfo()
    text:SetText(currentZone .. (currentInstanceDifficulty > 0 and ' ' .. GetDifficultyInfo(currentInstanceDifficulty) or ''))
    return true
end

function refreshSubzone(text)
    local currentZone = GetRealZoneText()
    local currentSubzone = GetSubZoneText()
    if currentZone ~= currentSubzone then
        text:SetText(currentSubzone)
        return true
    end
    return false
end

function refreshTimestamp(text)
    text:SetText(date('%B %d, %Y %I:%M%p'))
    return true
end

local textParts = {
    {name="name", text=createFrameText('QuestTitleFont'), refresh=refreshName},
    {name="guild", text=createFrameText('QuestFont'), refresh=refreshGuild},
    {name="level", text=createFrameText('QuestFont'), refresh=refreshLevel},
    {name="zone", text=createFrameText('QuestTitleFont'), refresh=refreshZone},
    {name="subzone", text=createFrameText('QuestFont'), refresh=refreshSubzone},
    {name="timestamp", text=createFrameText('QuestFont'), refresh=refreshTimestamp}
}

SLASH_KeepSake1 = '/KeepSake'
SLASH_KeepSake2 = '/ks'
SlashCmdList['KeepSake'] = function()
    UIParent:Hide()

    textParts[1].text:SetPoint('TOPLEFT', frame, 'TOPLEFT', 0, 0)
    textParts[1].refresh(textParts[1].text)
    local lastText = textParts[1].text
    for i = 2, 6 do
        local text = textParts[i].text
        local refresh = textParts[i].refresh
        if refresh(text) then
            -- add a small vertical gap between level and zone
            yOffset = 0
            if textParts[i].name == "zone" then
                yOffset = -framePaddingTop
            end
            text:SetPoint('TOPLEFT', lastText, 'BOTTOMLEFT', 0, yOffset)
            lastText = text
        else
            -- nothing useful displayed, clear out previous value (e.g. switching between alts)
            text:SetText(' ')
        end
    end

    frame:Show()
    Screenshot()
end

frame:RegisterEvent('SCREENSHOT_SUCCEEDED')
frame:RegisterEvent('SCREENSHOT_FAILED')
frame:SetScript('OnEvent', function(self, event, ...)
    if event == 'SCREENSHOT_SUCCEEDED' or event == 'SCREENSHOT_FAILED' then
        frame:Hide()
        UIParent:Show()
    end
end)
